@@ -1,9 +1,14 @@
 import datetime
+import importlib
 import logging
+import os
+import subprocess
 import sys
+import tempfile
 from contextlib import contextmanager, nullcontext
 from unittest.mock import patch
 
+import airflow
 import dateutil
 import lazy_object_proxy
 import pendulum
@@ -18,22 +23,77 @@ from dagster_airflow.patch_airflow_example_dag import patch_airflow_example_dag
 from dagster import (
     DagsterInvariantViolationError,
     DependencyDefinition,
+    Field,
     In,
+    JobDefinition,
     MultiDependencyDefinition,
     Nothing,
     Out,
+    ScheduleDefinition,
 )
 from dagster import _check as check
-from dagster import op, repository
+from dagster import op, repository, resource
 from dagster._core.definitions.utils import VALID_NAME_REGEX, validate_tags
 from dagster._core.instance import AIRFLOW_EXECUTION_DATE_STR, IS_AIRFLOW_INGEST_PIPELINE_STR
-from dagster._legacy import PipelineDefinition, SolidDefinition
+from dagster._legacy import ModeDefinition, PipelineDefinition, SolidDefinition
+from dagster._utils.schedules import is_valid_cron_schedule
+
+# pylint: disable=no-name-in-module,import-error
+if str(airflow_version) >= "2.0.0":
+    from airflow.utils.state import DagRunState
+    from airflow.utils.types import DagRunType
+else:
+    from airflow.utils.state import State
+# pylint: enable=no-name-in-module,import-error
 
 
 class DagsterAirflowError(Exception):
     pass
 
 
+if os.name == "nt":
+    import msvcrt  # pylint: disable=import-error
+
+    def portable_lock(fp):
+        fp.seek(0)
+        msvcrt.locking(fp.fileno(), msvcrt.LK_LOCK, 1)
+
+    def portable_unlock(fp):
+        fp.seek(0)
+        msvcrt.locking(fp.fileno(), msvcrt.LK_UNLCK, 1)
+
+else:
+    import fcntl
+
+    def portable_lock(fp):
+        fcntl.flock(fp.fileno(), fcntl.LOCK_EX)
+
+    def portable_unlock(fp):
+        fcntl.flock(fp.fileno(), fcntl.LOCK_UN)
+
+
+class Locker:
+    def __init__(self, lock_file_path="."):
+        self.lock_file_path = lock_file_path
+        self.fp = None
+
+    def __enter__(self):
+        self.fp = open(f"{self.lock_file_path}/lockfile.lck", "w+", encoding="utf-8")
+        portable_lock(self.fp)
+
+    def __exit__(self, _type, value, tb):
+        portable_unlock(self.fp)
+        self.fp.close()
+
+
+def initialize_airflow_1_database():
+    subprocess.run(["airflow", "initdb"], check=True)
+
+
+def initialize_airflow_2_database():
+    subprocess.run(["airflow", "db", "init"], check=True)
+
+
 def contains_duplicate_task_names(dag_bag, refresh_from_airflow_db):
     check.inst_param(dag_bag, "dag_bag", DagBag)
     check.bool_param(refresh_from_airflow_db, "refresh_from_airflow_db")
@@ -56,6 +116,8 @@ def make_dagster_repo_from_airflow_dag_bag(
     repo_name,
     refresh_from_airflow_db=False,
     use_airflow_template_context=False,
+    mock_xcom=False,
+    use_ephemeral_airflow_db=False,
 ):
     """Construct a Dagster repository corresponding to Airflow DAGs in DagBag.
 
@@ -77,8 +139,13 @@ def make_dagster_repo_from_airflow_dag_bag(
             which requires access to initialized Airflow DB. If False (recommended), gets dag from
             DagBag's dags dict without depending on Airflow DB. (default: False)
         use_airflow_template_context (bool): If True, will call get_template_context() on the
-            Airflow TaskInstance model which requires and modifies the DagRun table.
+            Airflow TaskInstance model which requires and modifies the DagRun table. The use_airflow_template_context
+            setting is ignored if use_ephemeral_airflow_db is True.
             (default: False)
+        mock_xcom (bool): If True, dagster will mock out all calls made to xcom, features that
+            depend on xcom may not work as expected. (default: False)
+        use_ephemeral_airflow_db (bool): If True, dagster will create an ephemeral sqlite airflow
+            database for each run. (default: False)
 
     Returns:
         RepositoryDefinition
@@ -87,10 +154,16 @@ def make_dagster_repo_from_airflow_dag_bag(
     check.str_param(repo_name, "repo_name")
     check.bool_param(refresh_from_airflow_db, "refresh_from_airflow_db")
     check.bool_param(use_airflow_template_context, "use_airflow_template_context")
+    mock_xcom = check.opt_bool_param(mock_xcom, "mock_xcom")
+    use_ephemeral_airflow_db = check.opt_bool_param(
+        use_ephemeral_airflow_db, "use_ephemeral_airflow_db"
+    )
 
     use_unique_id = contains_duplicate_task_names(dag_bag, refresh_from_airflow_db)
 
-    pipeline_defs = []
+    job_defs = []
+    schedule_defs = []
+    asset_defs = []
     count = 0
     # To enforce predictable iteration order
     sorted_dag_ids = sorted(dag_bag.dag_ids)
@@ -98,32 +171,95 @@ def make_dagster_repo_from_airflow_dag_bag(
         # Only call Airflow DB via dag_bag.get_dag(dag_id) if refresh_from_airflow_db is True
         dag = dag_bag.dags.get(dag_id) if not refresh_from_airflow_db else dag_bag.get_dag(dag_id)
         if not use_unique_id:
-            pipeline_defs.append(
-                make_dagster_pipeline_from_airflow_dag(
-                    dag=dag,
-                    tags=None,
-                    use_airflow_template_context=use_airflow_template_context,
-                )
+            pipeline_def = make_dagster_pipeline_from_airflow_dag(
+                dag=dag,
+                tags=None,
+                use_airflow_template_context=use_airflow_template_context,
+                mock_xcom=mock_xcom,
+                use_ephemeral_airflow_db=use_ephemeral_airflow_db,
             )
         else:
-            pipeline_defs.append(
-                make_dagster_pipeline_from_airflow_dag(
-                    dag=dag,
-                    tags=None,
-                    use_airflow_template_context=use_airflow_template_context,
-                    unique_id=count,
-                )
+            pipeline_def = make_dagster_pipeline_from_airflow_dag(
+                dag=dag,
+                tags=None,
+                use_airflow_template_context=use_airflow_template_context,
+                unique_id=count,
+                mock_xcom=mock_xcom,
+                use_ephemeral_airflow_db=use_ephemeral_airflow_db,
             )
             count += 1
+        # pass in tags manually because pipeline_def.graph doesn't have it threaded
+        job_def = pipeline_def.graph.to_job(
+            tags={**pipeline_def.tags},
+            resource_defs={
+                "airflow_db": pipeline_def.mode_definitions[0].resource_defs["airflow_db"]
+            }
+            if use_ephemeral_airflow_db
+            else {},
+        )
+        schedule_def = make_dagster_schedule_from_airflow_dag(
+            dag=dag,
+            job_def=job_def,
+        )
+        asset_def = make_dagster_asset_from_airflow_dag(dag=dag, job_def=job_def)
+        if schedule_def:
+            schedule_defs.append(schedule_def)
+        elif asset_def:
+            asset_defs.append(asset_def)
+        else:
+            job_defs.append(job_def)
 
     @repository(name=repo_name)
     def _repo():
-        return pipeline_defs
+        return [job_defs, schedule_defs]
 
     return _repo
 
 
-def make_dagster_repo_from_airflow_example_dags(repo_name="airflow_example_dags_repo"):
+def make_dagster_schedule_from_airflow_dag(dag, job_def):
+    """Construct a Dagster schedule corresponding to an Airflow DAG.
+
+    Args:
+        dag (DAG): Airflow DAG
+        job_def (JobDefinition): Dagster pipeline corresponding to Airflow DAG
+
+    Returns:
+        ScheduleDefinition
+    """
+    check.inst_param(dag, "dag", DAG)
+    check.inst_param(job_def, "job_def", JobDefinition)
+
+    cron_schedule = dag.normalized_schedule_interval
+    schedule_description = dag.description
+
+    if isinstance(dag.normalized_schedule_interval, str) and is_valid_cron_schedule(cron_schedule):
+        return ScheduleDefinition(
+            job=job_def, cron_schedule=cron_schedule, description=schedule_description
+        )
+
+
+def make_dagster_asset_from_airflow_dag(dag, job_def):
+    """Construct a Dagster asset corresponding to an Airflow DAG.
+
+    Args:
+        dag (DAG): Airflow DAG
+        job_def (JobDefinition): Dagster pipeline corresponding to Airflow DAG
+
+    Returns:
+        AssetDefinition
+    """
+    check.inst_param(dag, "dag", DAG)
+    check.inst_param(job_def, "job_def", JobDefinition)
+
+    cron_schedule = dag.normalized_schedule_interval
+    if isinstance(dag.normalized_schedule_interval, str) and cron_schedule == "Dataset":
+        # TODO: add support for asset tags
+        return
+
+
+def make_dagster_repo_from_airflow_example_dags(
+    repo_name="airflow_example_dags_repo", use_ephemeral_airflow_db=True
+):
     """Construct a Dagster repository for Airflow's example DAGs.
 
     Execution of the following Airflow example DAGs is not currently supported:
@@ -147,6 +283,8 @@ def make_dagster_repo_from_airflow_example_dags(repo_name="airflow_example_dags_
 
     Args:
         repo_name (str): Name for generated RepositoryDefinition
+        use_ephemeral_airflow_db (bool): If True, dagster will create an ephemeral sqlite airflow
+            database for each run. (default: False)
 
     Returns:
         RepositoryDefinition
@@ -156,13 +294,13 @@ def make_dagster_repo_from_airflow_example_dags(repo_name="airflow_example_dags_
         include_examples=True,
     )
 
-    # There is a bug in Airflow v1.10.8, v1.10.9, v1.10.10 where the python_callable for task
-    # 'search_catalog' is missing a required position argument '_'. It is currently fixed in master.
-    # v1.10 stable: https://github.com/apache/airflow/blob/v1-10-stable/airflow/example_dags/example_complex.py#L133
-    # master (05-05-2020): https://github.com/apache/airflow/blob/master/airflow/example_dags/example_complex.py#L136
+    # There is a bug in Airflow v1 where the python_callable for task
+    # 'search_catalog' is missing a required position argument '_'. It is fixed in airflow v2
     patch_airflow_example_dag(dag_bag)
 
-    return make_dagster_repo_from_airflow_dag_bag(dag_bag, repo_name)
+    return make_dagster_repo_from_airflow_dag_bag(
+        dag_bag, repo_name, use_ephemeral_airflow_db=use_ephemeral_airflow_db
+    )
 
 
 def make_dagster_repo_from_airflow_dags_path(
@@ -171,6 +309,8 @@ def make_dagster_repo_from_airflow_dags_path(
     safe_mode=True,
     store_serialized_dags=False,
     use_airflow_template_context=False,
+    mock_xcom=False,
+    use_ephemeral_airflow_db=True,
 ):
     """Construct a Dagster repository corresponding to Airflow DAGs in dag_path.
 
@@ -200,8 +340,13 @@ def make_dagster_repo_from_airflow_dags_path(
         store_serialized_dags (bool): True to read Airflow DAGS from Airflow DB. False to read DAGS
             from Python files. (default: False)
         use_airflow_template_context (bool): If True, will call get_template_context() on the
-            Airflow TaskInstance model which requires and modifies the DagRun table.
+            Airflow TaskInstance model which requires and modifies the DagRun table. The use_airflow_template_context
+            setting is ignored if use_ephemeral_airflow_db is True.
             (default: False)
+        mock_xcom (bool): If True, dagster will mock out all calls made to xcom, features that
+            depend on xcom may not work as expected. (default: False)
+        use_ephemeral_airflow_db (bool): If True, dagster will create an ephemeral sqlite airflow
+            database for each run. (default: False)
 
     Returns:
         RepositoryDefinition
@@ -211,22 +356,35 @@ def make_dagster_repo_from_airflow_dags_path(
     check.bool_param(safe_mode, "safe_mode")
     check.bool_param(store_serialized_dags, "store_serialized_dags")
     check.bool_param(use_airflow_template_context, "use_airflow_template_context")
-
+    mock_xcom = check.opt_bool_param(mock_xcom, "mock_xcom")
+    use_ephemeral_airflow_db = check.opt_bool_param(
+        use_ephemeral_airflow_db, "use_ephemeral_airflow_db"
+    )
     try:
         dag_bag = DagBag(
             dag_folder=dag_path,
             include_examples=False,  # Exclude Airflow example dags
             safe_mode=safe_mode,
-            store_serialized_dags=store_serialized_dags,
         )
     except Exception:
         raise DagsterAirflowError("Error initializing airflow.models.dagbag object with arguments")
 
-    return make_dagster_repo_from_airflow_dag_bag(dag_bag, repo_name, use_airflow_template_context)
+    return make_dagster_repo_from_airflow_dag_bag(
+        dag_bag,
+        repo_name,
+        use_airflow_template_context=use_airflow_template_context,
+        mock_xcom=mock_xcom,
+        use_ephemeral_airflow_db=use_ephemeral_airflow_db,
+    )
 
 
 def make_dagster_pipeline_from_airflow_dag(
-    dag, tags=None, use_airflow_template_context=False, unique_id=None, mock_xcom=False
+    dag,
+    tags=None,
+    use_airflow_template_context=False,
+    unique_id=None,
+    mock_xcom=False,
+    use_ephemeral_airflow_db=False,
 ):
     """Construct a Dagster pipeline corresponding to a given Airflow DAG.
 
@@ -272,12 +430,15 @@ def make_dagster_pipeline_from_airflow_dag(
             `tags={'airflow_execution_date': utc_date_string}` to specify execution_date used within
             execution of Airflow Operators.
         use_airflow_template_context (bool): If True, will call get_template_context() on the
-            Airflow TaskInstance model which requires and modifies the DagRun table.
+            Airflow TaskInstance model which requires and modifies the DagRun table. The use_airflow_template_context
+            setting is ignored if use_ephemeral_airflow_db is True.
             (default: False)
         unique_id (int): If not None, this id will be postpended to generated solid names. Used by
             framework authors to enforce unique solid names within a repo.
         mock_xcom (bool): If not None, dagster will mock out all calls made to xcom, features that
             depend on xcom may not work as expected.
+        use_ephemeral_airflow_db (bool): If True, dagster will create an ephemeral sqlite airflow
+            database for each run
 
     Returns:
         pipeline_def (PipelineDefinition): The generated Dagster pipeline
@@ -288,6 +449,9 @@ def make_dagster_pipeline_from_airflow_dag(
     check.bool_param(use_airflow_template_context, "use_airflow_template_context")
     unique_id = check.opt_int_param(unique_id, "unique_id")
     mock_xcom = check.opt_bool_param(mock_xcom, "mock_xcom")
+    use_ephemeral_airflow_db = check.opt_bool_param(
+        use_ephemeral_airflow_db, "use_ephemeral_airflow_db"
+    )
 
     if IS_AIRFLOW_INGEST_PIPELINE_STR not in tags:
         tags[IS_AIRFLOW_INGEST_PIPELINE_STR] = "true"
@@ -295,12 +459,67 @@ def make_dagster_pipeline_from_airflow_dag(
     tags = validate_tags(tags)
 
     pipeline_dependencies, solid_defs = _get_pipeline_definition_args(
-        dag, use_airflow_template_context, unique_id, mock_xcom
+        dag, use_airflow_template_context, unique_id, mock_xcom, use_ephemeral_airflow_db
     )
+
+    @resource(
+        config_schema={
+            "dag_location": Field(str, default_value=dag.fileloc),
+            "dag_id": Field(str, default_value=dag.dag_id),
+        }
+    )
+    def airflow_db(context):
+        airflow_home_path = os.path.join(tempfile.gettempdir(), f"dagster_airflow_{context.run_id}")
+        os.environ["AIRFLOW_HOME"] = airflow_home_path
+        os.makedirs(airflow_home_path, exist_ok=True)
+        with Locker(airflow_home_path):
+            airflow_initialized = os.path.exists(f"{airflow_home_path}/airflow.db")
+            if not airflow_initialized:
+                if airflow_version >= "2.0.0":
+                    initialize_airflow_2_database()
+                else:
+                    initialize_airflow_1_database()
+            # because AIRFLOW_HOME has been overriden airflow needs to be reloaded
+            if airflow_version >= "2.0.0":
+                importlib.reload(airflow.configuration)
+                importlib.reload(airflow.settings)
+                importlib.reload(airflow)
+            else:
+                importlib.reload(airflow)
+
+            dag_bag = airflow.models.dagbag.DagBag(
+                dag_folder=context.resource_config["dag_location"], include_examples=True
+            )
+            dag = dag_bag.get_dag(context.resource_config["dag_id"])
+            execution_date_str = context.dagster_run.tags.get(AIRFLOW_EXECUTION_DATE_STR)
+            execution_date = dateutil.parser.parse(execution_date_str)
+            dagrun = dag.get_dagrun(execution_date=execution_date)
+            if not dagrun:
+                if airflow_version >= "2.0.0":
+                    dagrun = dag.create_dagrun(
+                        state=DagRunState.RUNNING,
+                        execution_date=execution_date,
+                        run_type=DagRunType.MANUAL,
+                    )
+                else:
+                    dagrun = dag.create_dagrun(
+                        run_id=f"dagster_airflow_run_{execution_date}",
+                        state=State.RUNNING,
+                        execution_date=execution_date,
+                    )
+
+        return {
+            "dag": dag,
+            "dagrun": dagrun,
+        }
+
     pipeline_def = PipelineDefinition(
-        name=normalized_name(dag.dag_id, None),
+        name=normalized_name(dag.dag_id),
         solid_defs=solid_defs,
         dependencies=pipeline_dependencies,
+        mode_defs=[ModeDefinition(resource_defs={"airflow_db": airflow_db})]
+        if use_ephemeral_airflow_db
+        else [],
         tags=tags,
     )
     return pipeline_def
@@ -309,7 +528,7 @@ def make_dagster_pipeline_from_airflow_dag(
 # Airflow DAG ids and Task ids allow a larger valid character set (alphanumeric characters,
 # dashes, dots and underscores) than Dagster's naming conventions (alphanumeric characters,
 # underscores), so Dagster will strip invalid characters and replace with '_'
-def normalized_name(name, unique_id):
+def normalized_name(name, unique_id=None):
     base_name = "airflow_" + "".join(c if VALID_NAME_REGEX.match(c) else "_" for c in name)
     if not unique_id:
         return base_name
@@ -318,11 +537,19 @@ def normalized_name(name, unique_id):
 
 
 def _get_pipeline_definition_args(
-    dag, use_airflow_template_context, unique_id=None, mock_xcom=False
+    dag,
+    use_airflow_template_context,
+    unique_id=None,
+    mock_xcom=False,
+    use_ephemeral_airflow_db=False,
 ):
     check.inst_param(dag, "dag", DAG)
     check.bool_param(use_airflow_template_context, "use_airflow_template_context")
     unique_id = check.opt_int_param(unique_id, "unique_id")
+    mock_xcom = check.opt_bool_param(mock_xcom, "mock_xcom")
+    use_ephemeral_airflow_db = check.opt_bool_param(
+        use_ephemeral_airflow_db, "use_ephemeral_airflow_db"
+    )
 
     pipeline_dependencies = {}
     solid_defs = []
@@ -332,6 +559,7 @@ def _get_pipeline_definition_args(
     dag_roots = sorted(dag.roots, key=lambda x: x.task_id)
     for task in dag_roots:
         _traverse_airflow_dag(
+            dag,
             task,
             seen_tasks,
             pipeline_dependencies,
@@ -339,11 +567,13 @@ def _get_pipeline_definition_args(
             use_airflow_template_context,
             unique_id,
             mock_xcom,
+            use_ephemeral_airflow_db,
         )
     return (pipeline_dependencies, solid_defs)
 
 
 def _traverse_airflow_dag(
+    dag,
     task,
     seen_tasks,
     pipeline_dependencies,
@@ -351,17 +581,22 @@ def _traverse_airflow_dag(
     use_airflow_template_context,
     unique_id,
     mock_xcom,
+    use_ephemeral_airflow_db,
 ):
+    check.inst_param(dag, "dag", DAG)
     check.inst_param(task, "task", BaseOperator)
     check.list_param(seen_tasks, "seen_tasks", BaseOperator)
     check.list_param(solid_defs, "solid_defs", SolidDefinition)
     check.bool_param(use_airflow_template_context, "use_airflow_template_context")
     unique_id = check.opt_int_param(unique_id, "unique_id")
     mock_xcom = check.opt_bool_param(mock_xcom, "mock_xcom")
+    use_ephemeral_airflow_db = check.opt_bool_param(
+        use_ephemeral_airflow_db, "use_ephemeral_airflow_db"
+    )
 
     seen_tasks.append(task)
     current_solid = make_dagster_solid_from_airflow_task(
-        task, use_airflow_template_context, unique_id, mock_xcom
+        dag, task, use_airflow_template_context, unique_id, mock_xcom, use_ephemeral_airflow_db
     )
     solid_defs.append(current_solid)
 
@@ -386,6 +621,7 @@ def _traverse_airflow_dag(
     for child_task in task_downstream_list:
         if child_task not in seen_tasks:
             _traverse_airflow_dag(
+                dag,
                 child_task,
                 seen_tasks,
                 pipeline_dependencies,
@@ -393,6 +629,7 @@ def _traverse_airflow_dag(
                 use_airflow_template_context,
                 unique_id,
                 mock_xcom,
+                use_ephemeral_airflow_db,
             )
 
 
@@ -421,18 +658,35 @@ def _mock_xcom():
 # If unique_id is not None, this id will be postpended to generated solid names, generally used
 # to enforce unique solid names within a repo.
 def make_dagster_solid_from_airflow_task(
-    task, use_airflow_template_context, unique_id=None, mock_xcom=False
+    dag,
+    task,
+    use_airflow_template_context,
+    unique_id=None,
+    mock_xcom=False,
+    use_ephemeral_airflow_db=False,
 ):
+    check.inst_param(dag, "dag", DAG)
     check.inst_param(task, "task", BaseOperator)
     check.bool_param(use_airflow_template_context, "use_airflow_template_context")
     unique_id = check.opt_int_param(unique_id, "unique_id")
+    mock_xcom = check.opt_bool_param(mock_xcom, "mock_xcom")
+    use_ephemeral_airflow_db = check.opt_bool_param(
+        use_ephemeral_airflow_db, "use_ephemeral_airflow_db"
+    )
 
     @op(
         name=normalized_name(task.task_id, unique_id),
+        required_resource_keys={"airflow_db"} if use_ephemeral_airflow_db else None,
         ins={"airflow_task_ready": In(Nothing)},
         out={"airflow_task_complete": Out(Nothing)},
+        config_schema={
+            "mock_xcom": Field(bool, default_value=mock_xcom),
+            "use_ephemeral_airflow_db": Field(bool, default_value=use_ephemeral_airflow_db),
+        },
     )
     def _solid(context):  # pylint: disable=unused-argument
+        mock_xcom = context.op_config["mock_xcom"]
+        use_ephemeral_airflow_db = context.op_config["use_ephemeral_airflow_db"]
         if AIRFLOW_EXECUTION_DATE_STR not in context.pipeline_run.tags:
             raise DagsterInvariantViolationError(
                 'Could not find "{AIRFLOW_EXECUTION_DATE_STR}" in {target} tags "{tags}". Please '
@@ -463,23 +717,45 @@ def make_dagster_solid_from_airflow_task(
 
         check.inst_param(execution_date, "execution_date", datetime.datetime)
 
-        with _mock_xcom() if mock_xcom else nullcontext():
+        with _mock_xcom() if mock_xcom and not use_ephemeral_airflow_db else nullcontext():
             with replace_airflow_logger_handlers():
                 if airflow_version >= "2.0.0":
-                    task_instance = TaskInstance(
-                        task=task, execution_date=execution_date, run_id="dagster_airflow_run"
-                    )
+                    if use_ephemeral_airflow_db:
+                        dag = context.resources.airflow_db["dag"]
+                        dagrun = context.resources.airflow_db["dagrun"]
+                        ti = dagrun.get_task_instance(task_id=task.task_id)
+                        ti.task = dag.get_task(task_id=task.task_id)
+                        ti.run(ignore_ti_state=True)
+                    else:
+                        # the airflow db is not initialized so no dagrun or task instance exists
+                        ti = TaskInstance(
+                            task=task,
+                            execution_date=execution_date,
+                            run_id=f"dagster_airflow_run_{execution_date}",
+                        )
+                        ti_context = (
+                            dagster_get_template_context(ti, task, execution_date)
+                            if not use_airflow_template_context
+                            else ti.get_template_context()
+                        )
+                        task.render_template_fields(ti_context)
+                        task.execute(ti_context)
                 else:
-                    task_instance = TaskInstance(task=task, execution_date=execution_date)
-                ti_context = (
-                    dagster_get_template_context(task_instance, task, execution_date)
-                    if not use_airflow_template_context
-                    else task_instance.get_template_context()
-                )
-                task.render_template_fields(ti_context)
-
-                task.execute(ti_context)
-
+                    if use_ephemeral_airflow_db:
+                        dag = context.resources.airflow_db["dag"]
+                        dagrun = context.resources.airflow_db["dagrun"]
+                        ti = dagrun.get_task_instance(task_id=task.task_id)
+                        ti.task = dag.get_task(task_id=task.task_id)
+                        ti.run(ignore_ti_state=True)
+                    else:
+                        ti = TaskInstance(task=task, execution_date=execution_date)
+                        ti_context = (
+                            dagster_get_template_context(ti, task, execution_date)
+                            if not use_airflow_template_context
+                            else ti.get_template_context()
+                        )
+                        task.render_template_fields(ti_context)
+                        task.execute(ti_context)
                 return None
 
     return _solid
